#include <cmath>

#include <string>
#include <fstream>
#include <iostream>
#include <sstream>

#include <direct.h>

#include "Umgebung.h"

#include "Tools.h"
#include "Strategies.h"

using namespace std;

double DJ_B[TT_];
double DJ_Returns_B[TT_];
double OBVI_Returns_B[TT_];
double RF_B[TT_];
double OBVI_B[TT_];
int SSS_B[nb_strats_][N_];

double Perfs_tmp[nb_perf_measures_][nb_strats_];

double ReturnsOneStrat[N_];
double ReturnsOneStrat_SR[N_];
double ReturnsOneStrat_B[N_];
double ReturnsOneStrat_SR_B[N_];

double SR_ReturnsPorfolio[N_];
double mu_ReturnsPorfolio[N_];

double RF_bis[N_];
double RF_bis_B[N_];	//because of bootstrap, we truncate beginning of RF

double pvalues_bis[nb_perf_measures_][nb_strats_];

void ComputeReturnsOneStrat(int SS[], long R, long T, double TC, double LF, double DJ_Returns[], double RF[], double ReturnsOneStrat[], double ReturnsOneStrat_SR[], double RF_bis[]) {
	
	//input: SS, R, T, TC, LF, DJ_Returns, RF
	//output:	ReturnsOneStrat		log returns
	//			ReturnsOneStrat_SR	arith
	//			RF_bis				arith

	long t;
	int S, oldS;
	double today_return, today_return_SR;

	S=0;
	for(t=R-1; t<T; t++) {
		oldS=S;
		S=SS[t];
		today_return=DJ_Returns[t+1]*S;
		if(S!=oldS){
			if(S*oldS==-1)
				today_return-=2*TC;
			else if(S*oldS==0)
				today_return-=TC;
		}
		if(S==-1)
			today_return-=LF;
		
		if(S==0)
			today_return_SR=today_return+RF[t+1];
		else
			today_return_SR=today_return;

		ReturnsOneStrat[t-(R-1)]=log(1+today_return);
		ReturnsOneStrat_SR[t-(R-1)]=today_return_SR;
		RF_bis[t-(R-1)]=RF[t+1];
	}
}

void ComputeReturnsOneStrat(int SS[], long R, long T, double TC, double LF, double DJ_Returns[], double RF[], double ReturnsOneStrat[], double ReturnsOneStrat_SR[], double RF_bis[], double * nbshort) {
	
	//input: SS, R, T, TC, LF, DJ_Returns, RF
	//output:	ReturnsOneStrat		log returns
	//			ReturnsOneStrat_SR	arith
	//			RF_bis				arith

	long t;
	int S, oldS;
	double today_return, today_return_SR;
	double nbshort_tmp=0;

	S=0;
	for(t=R-1; t<T; t++) {
		oldS=S;
		S=SS[t];
		today_return=DJ_Returns[t+1]*S;
		if(S!=oldS){
			if(S*oldS==-1) {
				today_return-=2*TC;
			} else if(S*oldS==0) {
				today_return-=TC;
			}
		}
		if(S==-1) {
			today_return-=LF;
			nbshort_tmp++;
		}
		
		if(S==0) {
			today_return_SR=today_return+RF[t+1];
		} else {
			today_return_SR=today_return;
		}

		ReturnsOneStrat[t-(R-1)]=log(1+today_return);
		ReturnsOneStrat_SR[t-(R-1)]=today_return_SR;
		RF_bis[t-(R-1)]=RF[t+1];
	}
	*nbshort=nbshort_tmp;
}

void ComputeReturnsOneStrat_noshort(int SS[], long R, long T, double TC, double LF, double DJ_Returns[], double RF[], double ReturnsOneStrat[], double ReturnsOneStrat_SR[], double RF_bis[], double * nbshort) {
	
	//input: SS, R, T, TC, LF, DJ_Returns, RF
	//output:	ReturnsOneStrat		log returns
	//			ReturnsOneStrat_SR	arith
	//			RF_bis				arith

	long t;
	int S, oldS;
	double today_return, today_return_SR;
	double nbshort_tmp=0;

	double N=T-R+1;

	S=0;
	for(t=R-1; t<T; t++) {
		oldS=S;
		S=SS[t];
		if(S==0) {
			today_return=0;
			if(oldS==1) {
				today_return-=TC;
			}
		} else if(S==-1) {
			nbshort_tmp++;
			today_return=0;
			if(oldS==1) {
				today_return-=TC;
			}
		}  else if(S==1) {
			today_return=DJ_Returns[t+1];
			if(oldS!=1) {
				today_return-=TC;
			}
		}
		
		if(S!=1) {
			today_return_SR=today_return+RF[t+1];
		} else {
			today_return_SR=today_return;
		}

		ReturnsOneStrat[t-(R-1)]=log(1+today_return);
		ReturnsOneStrat_SR[t-(R-1)]=today_return_SR;
		RF_bis[t-(R-1)]=RF[t+1];
	}
	*nbshort=nbshort_tmp;
}


void ComputeReturnsOneStrat(int SS[], long R, long T, double TC, double LF, double DJ_Returns[], double RF[], double ReturnsOneStrat_SR[]) {

	double ignore1[N_];
	double ignore2[N_];

	ComputeReturnsOneStrat(SS, R, T, TC, LF, DJ_Returns, RF, ignore1, ReturnsOneStrat_SR, ignore2);
}


void ComputeReturnsOneStrat(int SS[], long R, long T, double TC, double LF, double DJ_Returns[], double RF[], int NBER[], double ReturnsOneStrat[], double ReturnsOneStrat_SR[], double RF_bis[], 
							double ReturnsOneStrat_exp[], double ReturnsOneStrat_SR_exp[], double RF_bis_exp[], long * N_exp, 
							double ReturnsOneStrat_cont[], double ReturnsOneStrat_SR_cont[], double RF_bis_cont[], long * N_cont) {
	
	//input: SS, R, T, TC, LF
	//output:	ReturnsOneStrat		log returns
	//			ReturnsOneStrat_SR	arith
	//			RF_bis				arith

	long t;
	int S, oldS;
	double today_return, today_return_SR;
	long t_exp=0;
	long t_cont=0;

	S=0;
	for(t=R-1; t<T; t++) {
		oldS=S;
		S=SS[t];
		today_return=DJ_Returns[t+1]*S;
		if(S!=oldS){
			if(S*oldS==-1)
				today_return-=2*TC;
			else if(S*oldS==0)
				today_return-=TC;
		}
		if(S==-1)
			today_return-=LF;
		
		if(S==0)
			today_return_SR=today_return+RF[t+1];
		else
			today_return_SR=today_return;

		ReturnsOneStrat[t-(R-1)]=log(1+today_return);
		ReturnsOneStrat_SR[t-(R-1)]=today_return_SR;
		RF_bis[t-(R-1)]=RF[t+1];

		if(NBER[t]==1) {
			ReturnsOneStrat_exp[t_exp]=ReturnsOneStrat[t-(R-1)];
			ReturnsOneStrat_SR_exp[t_exp]=ReturnsOneStrat_SR[t-(R-1)];
			RF_bis_exp[t_exp]=RF[t+1];
			t_exp++;
		} else {
			ReturnsOneStrat_cont[t_cont]=ReturnsOneStrat[t-(R-1)];
			ReturnsOneStrat_SR_cont[t_cont]=ReturnsOneStrat_SR[t-(R-1)];
			RF_bis_cont[t_cont]=RF[t+1];
			t_cont++;
		}
	}
	*N_exp=t_exp;
	*N_cont=t_cont;
}


void ComputeReturnsOneStrat(int SS[], long R, long T, double TC, double LF, double DJ_Returns[], double RF[], int NBER[], double ReturnsOneStrat_SR[], double ReturnsOneStrat_SR_exp[], long * N_exp, 
							double ReturnsOneStrat_SR_cont[], long * N_cont) {

	double ignore1[N_];
	double ignore2[N_];
	double ignore3[N_];
	double ignore4[N_];
	double ignore5[N_];
	double ignore6[N_];

	ComputeReturnsOneStrat(SS, R, T, TC, LF, DJ_Returns, RF, NBER, ignore1, ReturnsOneStrat_SR, ignore2, 
							ignore3, ReturnsOneStrat_SR_exp, ignore4, N_exp, 
							ignore5, ReturnsOneStrat_SR_cont, ignore6, N_cont);
}





void ComputeSR_ReturnsPorfolio_ptc_b(int SSS[][N_], long R, long T, double TC, double LF, double DJ_Returns[], 
									 double RF[], int Portfolio[], double oldproppos_init, double oldpropneg_init,
									 double SR_ReturnsPorfolio[], double * proppos_last, double * propneg_last, double RF_bis[]) {
	
	// _b: petite modif pour pooler correctement lors du rebalancing
	// _ptc: pooled transaction costs

	//string File_path;
	//ofstream propposFile, propnegFile, srrets2File, nbpospooledFile;
	//File_path=boulot_path_+"\\DATA\\dataprojet1\\resultats\\proppos.txt";
	//propposFile.open(File_path.c_str(), ofstream::out | ofstream::trunc);
	//File_path=boulot_path_+"\\DATA\\dataprojet1\\resultats\\propneg.txt";
	//propnegFile.open(File_path.c_str(), ofstream::out | ofstream::trunc);
	//File_path=boulot_path_+"\\DATA\\dataprojet1\\resultats\\srrets2.txt";
	//srrets2File.open(File_path.c_str(), ofstream::out | ofstream::trunc);
	//File_path=boulot_path_+"\\DATA\\dataprojet1\\resultats\\nbpospooled.txt";
	//nbpospooledFile.open(File_path.c_str(), ofstream::out | ofstream::trunc);

	long N=T-R+1;
	long t;
	int s;

	//int nbtransac; // >0 if buy, <0 if sell

	double nbpos, nbneutral, nbneg;
	double proppos, propneg, propneutral;
	double oldproppos, oldpropneg;
	double nbpos_pooled, nbneg_pooled, nbneutral_pooled;

	int portfolio_size=0;
	for(s=0; s<nb_strats_; s++) {
		if(Portfolio[s]==1) {
			portfolio_size++;
		}
	}

	// portfolio before rebalancing:
	proppos=oldproppos_init;
	propneg=oldpropneg_init;

	if(portfolio_size>0) {
		for(t=R-1; t<T; t++) {
			oldproppos=proppos;
			oldpropneg=propneg;
			nbpos=0;
			nbneutral=0;
			nbneg=0;
			for(s=0; s<nb_strats_; s++) {
				if(Portfolio[s]==1) {
					if(SSS[s][t]==1) {
						nbpos++;
					} else if(SSS[s][t]==0) {
						nbneutral++;
					} else {
						nbneg++;
					}
				}
			}

			nbpos_pooled=max(0.0,nbpos-nbneg);
			nbneg_pooled=max(0.0,nbneg-nbpos);
			nbneutral_pooled=nbneutral+min(nbpos,nbneg);

			proppos=nbpos_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);
			propneg=nbneg_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);
			propneutral=nbneutral_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);

			SR_ReturnsPorfolio[t-(R-1)]=proppos*DJ_Returns[t+1]-propneg*DJ_Returns[t+1]+propneutral*RF[t+1]-
				(abs(oldproppos-proppos)+abs(oldpropneg-propneg))*TC-propneg*LF;
			//SR_ReturnsPorfolio[t-(R-1)]=proppos*DJ_Returns[t+1]-propneg*DJ_Returns[t+1]-
			//	(abs(oldproppos-proppos)+abs(oldpropneg-propneg))*TC-propneg*LF;

			//propposFile << proppos << endl;
			//propnegFile << propneg << endl;
			//srrets2File << SR_ReturnsPorfolio[t-(R-1)] << endl;
			//nbpospooledFile << nbpos_pooled << endl;

		}
		*proppos_last=proppos;
		*propneg_last=propneg;
	} else {
		for(t=R-1; t<T; t++) {
			SR_ReturnsPorfolio[t-(R-1)]=RF[t+1];
			//SR_ReturnsPorfolio[t-(R-1)]=0;
		}
		*proppos_last=oldproppos_init;
		*propneg_last=oldpropneg_init;
	}

	for(t=R-1; t<T; t++) {
		RF_bis[t-(R-1)]=RF[t+1];
	}

	//propposFile.close();
	//propnegFile.close();
	//srrets2File.close();
	//nbpospooledFile.close();
}

void ComputeSR_ReturnsPorfolio_ptc_b(int SSS[][N_], long R, long T, double TC, double LF, double DJ_Returns[], 
									 double RF[], int Portfolio[], double oldproppos_init, double oldpropneg_init,
									 double SR_ReturnsPorfolio[], double * proppos_last, double * propneg_last) {
	
	// _b: petite modif pour pooler correctement lors du rebalancing
	// _ptc: pooled transaction costs

	long N=T-R+1;
	long t;
	int s;

	//int nbtransac; // >0 if buy, <0 if sell

	double nbpos, nbneutral, nbneg;
	double proppos, propneg, propneutral;
	double oldproppos, oldpropneg;
	double nbpos_pooled, nbneg_pooled, nbneutral_pooled;

	int portfolio_size=0;
	for(s=0; s<nb_strats_; s++) {
		if(Portfolio[s]==1) {
			portfolio_size++;
		}
	}

	// portfolio before rebalancing:
	proppos=oldproppos_init;
	propneg=oldpropneg_init;

	if(portfolio_size>0) {
		for(t=R-1; t<T; t++) {
			oldproppos=proppos;
			oldpropneg=propneg;
			nbpos=0;
			nbneutral=0;
			nbneg=0;
			for(s=0; s<nb_strats_; s++) {
				if(Portfolio[s]==1) {
					if(SSS[s][t]==1) {
						nbpos++;
					} else if(SSS[s][t]==0) {
						nbneutral++;
					} else {
						nbneg++;
					}
				}
			}

			nbpos_pooled=max(0.0,nbpos-nbneg);
			nbneg_pooled=max(0.0,nbneg-nbpos);
			nbneutral_pooled=nbneutral+min(nbpos,nbneg);

			proppos=nbpos_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);
			propneg=nbneg_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);
			propneutral=nbneutral_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);

			SR_ReturnsPorfolio[t-(R-1)]=proppos*DJ_Returns[t+1]-propneg*DJ_Returns[t+1]+propneutral*RF[t+1]-
				(abs(oldproppos-proppos)+abs(oldpropneg-propneg))*TC-propneg*LF;
		}
		*proppos_last=proppos;
		*propneg_last=propneg;
	} else {
		for(t=R-1; t<T; t++) {
			SR_ReturnsPorfolio[t-(R-1)]=RF[t+1];
		}
		*proppos_last=oldproppos_init;
		*propneg_last=oldpropneg_init;
	}
}

void ComputeSR_ReturnsPorfolio_ptc_b_nbshort(int SSS[][N_], long R, long T, double TC, double LF, double DJ_Returns[], 
									 double RF[], int Portfolio[], double oldproppos_init, double oldpropneg_init,
									 double SR_ReturnsPorfolio[], double * proppos_last, double * propneg_last, double * nbshort) {
	
	// _b: petite modif pour pooler correctement lors du rebalancing
	// _ptc: pooled transaction costs

	long N=T-R+1;
	long t;
	int s;

	//int nbtransac; // >0 if buy, <0 if sell

	double nbpos, nbneutral, nbneg;
	double proppos, propneg, propneutral;
	double oldproppos, oldpropneg;
	double nbpos_pooled, nbneg_pooled, nbneutral_pooled;

	double nbshort_tmp=0;

	int portfolio_size=0;
	for(s=0; s<nb_strats_; s++) {
		if(Portfolio[s]==1) {
			portfolio_size++;
		}
	}

	// portfolio before rebalancing:
	proppos=oldproppos_init;
	propneg=oldpropneg_init;

	if(portfolio_size>0) {
		for(t=R-1; t<T; t++) {
			oldproppos=proppos;
			oldpropneg=propneg;
			nbpos=0;
			nbneutral=0;
			nbneg=0;
			for(s=0; s<nb_strats_; s++) {
				if(Portfolio[s]==1) {
					if(SSS[s][t]==1) {
						nbpos++;
					} else if(SSS[s][t]==0) {
						nbneutral++;
					} else {
						nbneg++;
					}
				}
			}

			nbpos_pooled=max(0.0,nbpos-nbneg);
			nbneg_pooled=max(0.0,nbneg-nbpos);
			nbneutral_pooled=nbneutral+min(nbpos,nbneg);

			proppos=nbpos_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);
			propneg=nbneg_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);
			propneutral=nbneutral_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);

			if(nbneg_pooled>0) {
				nbshort_tmp++;
			}

			SR_ReturnsPorfolio[t-(R-1)]=proppos*DJ_Returns[t+1]-propneg*DJ_Returns[t+1]+propneutral*RF[t+1]-
				(abs(oldproppos-proppos)+abs(oldpropneg-propneg))*TC-propneg*LF;
		}
		*proppos_last=proppos;
		*propneg_last=propneg;
		*nbshort=nbshort_tmp;
	} else {
		for(t=R-1; t<T; t++) {
			SR_ReturnsPorfolio[t-(R-1)]=RF[t+1];
		}
		*proppos_last=oldproppos_init;
		*propneg_last=oldpropneg_init;
		*nbshort=0;
	}
}

void ComputeSR_ReturnsPorfolio_ptc_noshort(int SSS[][N_], long R, long T, double TC, double LF, double DJ_Returns[], 
									 double RF[], int Portfolio[], double oldproppos_init,
									 double SR_ReturnsPorfolio[], double * proppos_last, double * nbshort) {
	
	// _b: petite modif pour pooler correctement lors du rebalancing
	// _ptc: pooled transaction costs

	long N=T-R+1;
	long t;
	int s;

	//int nbtransac; // >0 if buy, <0 if sell

	double nbpos, nbneutral, nbneg;
	double proppos, propneg, propneutral;
	double oldproppos;
	double nbpos_pooled, nbneg_pooled, nbneutral_pooled;

	double nbshort_tmp=0;

	int portfolio_size=0;
	for(s=0; s<nb_strats_; s++) {
		if(Portfolio[s]==1) {
			portfolio_size++;
		}
	}

	// portfolio before rebalancing:
	proppos=oldproppos_init;

	if(portfolio_size>0) {
		for(t=R-1; t<T; t++) {
			oldproppos=proppos;
			nbpos=0;
			nbneutral=0;
			nbneg=0;
			for(s=0; s<nb_strats_; s++) {
				if(Portfolio[s]==1) {
					if(SSS[s][t]==1) {
						nbpos++;
					} else if(SSS[s][t]==0) {
						nbneutral++;
					} else {
						nbneg++;
					}
				}
			}

			nbpos_pooled=max(0.0,nbpos-nbneg);
			nbneg_pooled=max(0.0,nbneg-nbpos);
			nbneutral_pooled=nbneutral+min(nbpos,nbneg);

			proppos=nbpos_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);
			propneg=nbneg_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);
			propneutral=nbneutral_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);

			if(nbneg_pooled>0) {
				nbshort_tmp++;
			}

			SR_ReturnsPorfolio[t-(R-1)]=proppos*DJ_Returns[t+1]+(propneutral+propneg)*RF[t+1]-
				abs(oldproppos-proppos)*TC;
		}
		*proppos_last=proppos;
		*nbshort=nbshort_tmp;
	} else {
		for(t=R-1; t<T; t++) {
			SR_ReturnsPorfolio[t-(R-1)]=RF[t+1];
		}
		*proppos_last=oldproppos_init;
		*nbshort=0;
	}
}



double ComputeSR_nbporttransacs(int SSS[][N_], long R, long T, int Portfolio[]) {

	long N=T-R+1;
	long t;
	int s;

	double nbporttransacs=0;

	double nbpos, nbneutral, nbneg;
	double proppos, propneg;
	double oldproppos, oldpropneg;
	double nbpos_pooled, nbneg_pooled, nbneutral_pooled;

	int portfolio_size=0;
	for(s=0; s<nb_strats_; s++) {
		if(Portfolio[s]==1) {
			portfolio_size++;
		}
	}

	proppos=0;
	propneg=0;

	if(portfolio_size>0) {
		for(t=R-1; t<T; t++) {
			oldproppos=proppos;
			oldpropneg=propneg;
			nbpos=0;
			nbneutral=0;
			nbneg=0;
			for(s=0; s<nb_strats_; s++) {
				if(Portfolio[s]==1) {
					if(SSS[s][t]==1) {
						nbpos++;
					} else if(SSS[s][t]==0) {
						nbneutral++;
					} else {
						nbneg++;
					}
				}
			}

			nbpos_pooled=max(0.0,nbpos-nbneg);
			nbneg_pooled=max(0.0,nbneg-nbpos);
			nbneutral_pooled=nbneutral+min(nbpos,nbneg);

			proppos=nbpos_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);
			propneg=nbneg_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);

			nbporttransacs+=abs(oldproppos-proppos)+abs(oldpropneg-propneg);
		}
	}

	return nbporttransacs;
}


void Computemu_ReturnsPorfolio_ptc_b(int SSS[][N_], long R, long T, double TC, double LF, double DJ_Returns[], 
									 int Portfolio[], double oldproppos_init, double oldpropneg_init,
									 double output_ReturnsPorfolio[], double * proppos_last, double * propneg_last) {
	
	// _b: petite modif pour pooler correctement lors du rebalancing
	// _ptc: pooled transaction costs

	long N=T-R+1;
	long t;
	int s;

	//int nbtransac; // >0 if buy, <0 if sell

	double nbpos, nbneutral, nbneg;
	double proppos, propneg;
	double oldproppos, oldpropneg;
	double nbpos_pooled, nbneg_pooled, nbneutral_pooled;

	int portfolio_size=0;
	for(s=0; s<nb_strats_; s++) {
		if(Portfolio[s]==1) {
			portfolio_size++;
		}
	}

	// portfolio before rebalancing:
	proppos=oldproppos_init;
	propneg=oldpropneg_init;

	if(portfolio_size>0) {
		for(t=R-1; t<T; t++) {
			oldproppos=proppos;
			oldpropneg=propneg;
			nbpos=0;
			nbneutral=0;
			nbneg=0;
			for(s=0; s<nb_strats_; s++) {
				if(Portfolio[s]==1) {
					if(SSS[s][t]==1) {
						nbpos++;
					} else if(SSS[s][t]==0) {
						nbneutral++;
					} else {
						nbneg++;
					}
				}
			}

			nbpos_pooled=max(0.0,nbpos-nbneg);
			nbneg_pooled=max(0.0,nbneg-nbpos);
			nbneutral_pooled=nbneutral+min(nbpos,nbneg);

			proppos=nbpos_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);
			propneg=nbneg_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);
			//propneutral=nbneutral_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);

			output_ReturnsPorfolio[t-(R-1)]=log(1+proppos*DJ_Returns[t+1]-propneg*DJ_Returns[t+1]-
				(abs(oldproppos-proppos)+abs(oldpropneg-propneg))*TC-propneg*LF);

		}
		*proppos_last=proppos;
		*propneg_last=propneg;
	} else {
		for(t=R-1; t<T; t++) {
			output_ReturnsPorfolio[t-(R-1)]=0;
		}
		*proppos_last=oldproppos_init;
		*propneg_last=oldpropneg_init;
	}
}

void Computemu_ReturnsPorfolio_ptc_noshort(int SSS[][N_], long R, long T, double TC, double LF, double DJ_Returns[], 
									 int Portfolio[], double oldproppos_init,
									 double output_ReturnsPorfolio[], double * proppos_last, double * nbshort) {
	
	// _b: petite modif pour pooler correctement lors du rebalancing
	// _ptc: pooled transaction costs

	long N=T-R+1;
	long t;
	int s;

	//int nbtransac; // >0 if buy, <0 if sell

	double nbpos, nbneutral, nbneg;
	double proppos;
	double oldproppos;
	double nbpos_pooled, nbneg_pooled, nbneutral_pooled;

	double nbshort_tmp=0;

	int portfolio_size=0;
	for(s=0; s<nb_strats_; s++) {
		if(Portfolio[s]==1) {
			portfolio_size++;
		}
	}

	// portfolio before rebalancing:
	proppos=oldproppos_init;

	if(portfolio_size>0) {
		for(t=R-1; t<T; t++) {
			oldproppos=proppos;
			nbpos=0;
			nbneutral=0;
			nbneg=0;
			for(s=0; s<nb_strats_; s++) {
				if(Portfolio[s]==1) {
					if(SSS[s][t]==1) {
						nbpos++;
					} else if(SSS[s][t]==0) {
						nbneutral++;
					} else {
						nbneg++;
					}
				}
			}

			nbpos_pooled=max(0.0,nbpos-nbneg);
			nbneg_pooled=max(0.0,nbneg-nbpos);
			nbneutral_pooled=nbneutral+min(nbpos,nbneg);

			proppos=nbpos_pooled/(nbpos_pooled+nbneg_pooled+nbneutral_pooled);

			if(nbneg_pooled>0) {
				nbshort_tmp++;
			}

			output_ReturnsPorfolio[t-(R-1)]=log(1+proppos*DJ_Returns[t+1]-
				abs(oldproppos-proppos)*TC);

		}
		*proppos_last=proppos;
		*nbshort=nbshort_tmp;
	} else {
		for(t=R-1; t<T; t++) {
			output_ReturnsPorfolio[t-(R-1)]=0;
		}
		*proppos_last=oldproppos_init;
		*nbshort=0;
	}
}


double SharpeRatio(double DJ_ReturnsOneStrat_SR[], double RF[], long N) {
	//returns annualized SR
	//RF[] needed and different from global variable
	long t;
	double SR;
	double mu=Sum(DJ_ReturnsOneStrat_SR, N)/N;

	double fe[TT_];

	if(mu==0) {
		SR=0;
	} else {
		double fe_mu=0;
		for(t=0; t<N; t++) {
			fe[t]=DJ_ReturnsOneStrat_SR[t]-RF[t];
			fe_mu+=fe[t];
		}
		fe_mu=fe_mu/N;
		double sigmae=0;
		for(t=0; t<N; t++) {
			sigmae+=pow(fe[t]-fe_mu, 2);
		}
		sigmae=sigmae/(N-1);
		sigmae=sqrt(sigmae);

		if(sigmae==0) {
			SR=0;
		} else {
			SR=sqrt((double)252)*fe_mu/sigmae;
			//252/sqrt((double)252) == sqrt((double)252)
		}
	}
	return SR;
}


double MeanReturn(double ReturnsOneStrat[], long N) {
	//annualized!!!!
	double mu=T_fin_year_*Sum(ReturnsOneStrat, N)/N;
	return mu;
}


double Portfolio_SR_ptc(int SSS[][N_], long R, long T, double TC, double LF, double DJ_Returns[], double RF[], int Portfolio[]) {
	
	// ptc: pooled transaction costs

	double SR;
	long N=T-R+1;

	double lastproppos, lastpropneg;

	if(Sum(Portfolio, nb_strats_)>0) {

		ComputeSR_ReturnsPorfolio_ptc_b(SSS, R, T, TC, LF, DJ_Returns, RF, Portfolio, 0, 0,
									 SR_ReturnsPorfolio, &lastproppos, &lastpropneg, RF_bis);

		SR=SharpeRatio(SR_ReturnsPorfolio, RF_bis, N);
	} else {
		SR=-1;
	}
	return SR;
}

double Portfolio_mu_ptc(int SSS[][N_], long R, long T, double TC, double LF, double DJ_Returns[], int Portfolio[]) {
	
	// ptc: pooled transaction costs

	double mu;
	long N=T-R+1;

	double lastproppos, lastpropneg;

	if(Sum(Portfolio, nb_strats_)>0) {
		Computemu_ReturnsPorfolio_ptc_b(SSS, R, T, TC, LF, DJ_Returns, Portfolio, 0, 0,
									 mu_ReturnsPorfolio, &lastproppos, &lastpropneg);
		mu=Sum(mu_ReturnsPorfolio, N)/N;
	} else {
		mu=-1;
	}
	return mu;
}



double StandardDeviation(double ReturnsOneStrat[], long N) {
	long t;
	double mu=Sum(ReturnsOneStrat, N)/N;
	double sigma=0;
	for(t=0; t<N; t++) {
		sigma+=pow(ReturnsOneStrat[t], 2);
	}
	sigma=sigma/N;
	sigma-=pow(mu, 2);
	sigma=sqrt(sigma);
	return sigma;
}

void MultiPerfs(int SSS[][N_], long R, long T, double TC, double LF, double DJ_Returns[], double RF[], double Perfs[][nb_strats_]) {

	long N=T-R+1;

	for(int s=0; s<nb_strats_; s++) {
		ComputeReturnsOneStrat(SSS[s], R, T, TC, LF, DJ_Returns, RF, ReturnsOneStrat, ReturnsOneStrat_SR, RF_bis);
		Perfs[0][s]=Sum(ReturnsOneStrat, N)/N;
		Perfs[1][s]=SharpeRatio(ReturnsOneStrat_SR, RF_bis, N);
	}
}

void MultiPerfs(int SSS[][N_], long R, long T, double TC, double LF, double DJ_Returns[], double RF[], double Perfs[][nb_strats_], double Sigmas[]) {

	long N=T-R+1;

	for(int s=0; s<nb_strats_; s++) {
		ComputeReturnsOneStrat(SSS[s], R, T, TC, LF, DJ_Returns, RF, ReturnsOneStrat, ReturnsOneStrat_SR, RF_bis);
		Perfs[0][s]=Sum(ReturnsOneStrat, N)/N;
		Perfs[1][s]=SharpeRatio(ReturnsOneStrat_SR, RF_bis, N);
		Sigmas[s]=StandardDeviation(ReturnsOneStrat, N);
	}
}

void MultiPerfs(int SSS[nb_strats_][N_], long R, long T, double TC, double LF, double DJ_Returns[TT_], double RF[TT_], 
				long IDX[B_][N_], double Perfs[nb_perf_measures_][nb_strats_], 
				double pvalues[nb_perf_measures_][nb_strats_], double Perfs_B[nb_perf_measures_][B_][nb_strats_]) {

	long N=T-R+1;

	long t;
	int b, s, i_perf;
	double mean_Perfs_B;

	//string File_path=boulot_path_+"\\DATA\\dataprojet1\\resultats\\retsstrats.txt";
	//ofstream RetsFile;
	//RetsFile.open(File_path.c_str(), ofstream::out | ofstream::trunc);
	////stringstream out;

	for(s=0; s<nb_strats_; s++) {
		ComputeReturnsOneStrat(SSS[s], R, T, TC, LF, DJ_Returns, RF, ReturnsOneStrat, ReturnsOneStrat_SR, RF_bis);
		
		////out << s;
		////File_path=boulot_path_+"\\DATA\\dataprojet1\\resultats\\retsstrat" + out.str() + ".txt";
		////RetsFile.open(File_path.c_str(), ofstream::out | ofstream::trunc);
		//for(t=0; t<N; t++) {
		//	RetsFile << ReturnsOneStrat_SR[t] << endl;
		//}
		////RetsFile.close();

		Perfs[0][s]=Sum(ReturnsOneStrat, N)/N;
		Perfs[1][s]=SharpeRatio(ReturnsOneStrat_SR, RF_bis, N);
		//Perfs[1][s]=SharpeRatio_b(ReturnsOneStrat_SR, RF_bis, R, T);
		for(b=0; b<B_; b++) {
			for(t=0; t<N; t++) {
				ReturnsOneStrat_B[t]=ReturnsOneStrat[IDX[b][t]];
				ReturnsOneStrat_SR_B[t]=ReturnsOneStrat_SR[IDX[b][t]];
				RF_bis_B[t]=RF_bis[IDX[b][t]];
			}
			Perfs_B[0][b][s]=Sum(ReturnsOneStrat_B, N)/N;
			Perfs_B[1][b][s]=SharpeRatio(ReturnsOneStrat_SR_B, RF_bis_B, N);
		}
		DispProgress(s);
	}

	//RetsFile.close();

	//Compute p-values
	for(s=0; s<nb_strats_; s++) {
		for(i_perf=0; i_perf<nb_perf_measures_; i_perf++) {
			pvalues[i_perf][s]=0;
			pvalues_bis[i_perf][s]=0;
		}
	}

	for(i_perf=0; i_perf<nb_perf_measures_; i_perf++) {
		for(s=0; s<nb_strats_; s++) {
			mean_Perfs_B=0;
			for(b=0; b<B_; b++) {
				mean_Perfs_B+=Perfs_B[i_perf][b][s];
			}
			mean_Perfs_B/=B_;
			for(b=0; b<B_; b++) {
				if((Perfs_B[i_perf][b][s]-mean_Perfs_B)>Perfs[i_perf][s]) {
					pvalues[i_perf][s]++;
				} else {
					pvalues_bis[i_perf][s]++;
				}
			}
			pvalues[i_perf][s]=(pvalues[i_perf][s]<pvalues_bis[i_perf][s]) ? 2*pvalues[i_perf][s]/B_ : 2*pvalues_bis[i_perf][s]/B_;
			if( (pvalues[i_perf][s]==0) && (Perfs[i_perf][s]==0) ) {
				pvalues[i_perf][s]=-1;
			}		
		}
	}

}

void MultiPerfs(int SSS[nb_strats_][N_], long R, long T, double TC, double LF, double DJ_Returns[TT_], double RF[TT_], 
				long IDX[B_][N_], double Perfs[nb_perf_measures_][nb_strats_], 
				double pvalues[nb_perf_measures_][nb_strats_], double Perfs_B[nb_perf_measures_][B_][nb_strats_], double NBshort[nb_strats_]) {

	long N=T-R+1;

	long t;
	int b, s, i_perf;
	double mean_Perfs_B;

	//string File_path=boulot_path_+"\\DATA\\dataprojet1\\resultats\\retsstrats.txt";
	//ofstream RetsFile;
	//RetsFile.open(File_path.c_str(), ofstream::out | ofstream::trunc);
	////stringstream out;

	for(s=0; s<nb_strats_; s++) {
		ComputeReturnsOneStrat(SSS[s], R, T, TC, LF, DJ_Returns, RF, ReturnsOneStrat, ReturnsOneStrat_SR, RF_bis, &NBshort[s]);
		
		////out << s;
		////File_path=boulot_path_+"\\DATA\\dataprojet1\\resultats\\retsstrat" + out.str() + ".txt";
		////RetsFile.open(File_path.c_str(), ofstream::out | ofstream::trunc);
		//for(t=0; t<N; t++) {
		//	RetsFile << ReturnsOneStrat_SR[t] << endl;
		//}
		////RetsFile.close();

		Perfs[0][s]=Sum(ReturnsOneStrat, N)/N;
		Perfs[1][s]=SharpeRatio(ReturnsOneStrat_SR, RF_bis, N);
		//Perfs[1][s]=SharpeRatio_b(ReturnsOneStrat_SR, RF_bis, R, T);
		for(b=0; b<B_; b++) {
			for(t=0; t<N; t++) {
				ReturnsOneStrat_B[t]=ReturnsOneStrat[IDX[b][t]];
				ReturnsOneStrat_SR_B[t]=ReturnsOneStrat_SR[IDX[b][t]];
				RF_bis_B[t]=RF_bis[IDX[b][t]];
			}
			Perfs_B[0][b][s]=Sum(ReturnsOneStrat_B, N)/N;
			Perfs_B[1][b][s]=SharpeRatio(ReturnsOneStrat_SR_B, RF_bis_B, N);
		}
		DispProgress(s);
	}

	//RetsFile.close();

	//Compute p-values
	for(s=0; s<nb_strats_; s++) {
		for(i_perf=0; i_perf<nb_perf_measures_; i_perf++) {
			pvalues[i_perf][s]=0;
			pvalues_bis[i_perf][s]=0;
		}
	}

	for(i_perf=0; i_perf<nb_perf_measures_; i_perf++) {
		for(s=0; s<nb_strats_; s++) {
			mean_Perfs_B=0;
			for(b=0; b<B_; b++) {
				mean_Perfs_B+=Perfs_B[i_perf][b][s];
			}
			mean_Perfs_B/=B_;
			for(b=0; b<B_; b++) {
				if((Perfs_B[i_perf][b][s]-mean_Perfs_B)>Perfs[i_perf][s]) {
					pvalues[i_perf][s]++;
				} else {
					pvalues_bis[i_perf][s]++;
				}
			}
			pvalues[i_perf][s]=(pvalues[i_perf][s]<pvalues_bis[i_perf][s]) ? 2*pvalues[i_perf][s]/B_ : 2*pvalues_bis[i_perf][s]/B_;
			if( (pvalues[i_perf][s]==0) && (Perfs[i_perf][s]==0) ) {
				pvalues[i_perf][s]=-1;
			}		
		}
	}

}


void MultiPerfs_noshort(int SSS[nb_strats_][N_], long R, long T, double TC, double LF, double DJ_Returns[TT_], double RF[TT_], 
				long IDX[B_][N_], double Perfs[nb_perf_measures_][nb_strats_], 
				double pvalues[nb_perf_measures_][nb_strats_], double Perfs_B[nb_perf_measures_][B_][nb_strats_], double NBshort[nb_strats_]) {

	long N=T-R+1;

	long t;
	int b, s, i_perf;
	double mean_Perfs_B;

	//string File_path=boulot_path_+"\\DATA\\dataprojet1\\resultats\\retsstrats.txt";
	//ofstream RetsFile;
	//RetsFile.open(File_path.c_str(), ofstream::out | ofstream::trunc);
	////stringstream out;

	for(s=0; s<nb_strats_; s++) {
		ComputeReturnsOneStrat_noshort(SSS[s], R, T, TC, LF, DJ_Returns, RF, ReturnsOneStrat, ReturnsOneStrat_SR, RF_bis, &NBshort[s]);
		
		////out << s;
		////File_path=boulot_path_+"\\DATA\\dataprojet1\\resultats\\retsstrat" + out.str() + ".txt";
		////RetsFile.open(File_path.c_str(), ofstream::out | ofstream::trunc);
		//for(t=0; t<N; t++) {
		//	RetsFile << ReturnsOneStrat_SR[t] << endl;
		//}
		////RetsFile.close();

		Perfs[0][s]=Sum(ReturnsOneStrat, N)/N;
		Perfs[1][s]=SharpeRatio(ReturnsOneStrat_SR, RF_bis, N);
		//Perfs[1][s]=SharpeRatio_b(ReturnsOneStrat_SR, RF_bis, R, T);
		for(b=0; b<B_; b++) {
			for(t=0; t<N; t++) {
				ReturnsOneStrat_B[t]=ReturnsOneStrat[IDX[b][t]];
				ReturnsOneStrat_SR_B[t]=ReturnsOneStrat_SR[IDX[b][t]];
				RF_bis_B[t]=RF_bis[IDX[b][t]];
			}
			Perfs_B[0][b][s]=Sum(ReturnsOneStrat_B, N)/N;
			Perfs_B[1][b][s]=SharpeRatio(ReturnsOneStrat_SR_B, RF_bis_B, N);
		}
		DispProgress(s);
	}

	//RetsFile.close();

	//Compute p-values
	for(s=0; s<nb_strats_; s++) {
		for(i_perf=0; i_perf<nb_perf_measures_; i_perf++) {
			pvalues[i_perf][s]=0;
			pvalues_bis[i_perf][s]=0;
		}
	}

	for(i_perf=0; i_perf<nb_perf_measures_; i_perf++) {
		for(s=0; s<nb_strats_; s++) {
			mean_Perfs_B=0;
			for(b=0; b<B_; b++) {
				mean_Perfs_B+=Perfs_B[i_perf][b][s];
			}
			mean_Perfs_B/=B_;
			for(b=0; b<B_; b++) {
				if((Perfs_B[i_perf][b][s]-mean_Perfs_B)>Perfs[i_perf][s]) {
					pvalues[i_perf][s]++;
				} else {
					pvalues_bis[i_perf][s]++;
				}
			}
			pvalues[i_perf][s]=(pvalues[i_perf][s]<pvalues_bis[i_perf][s]) ? 2*pvalues[i_perf][s]/B_ : 2*pvalues_bis[i_perf][s]/B_;
			if( (pvalues[i_perf][s]==0) && (Perfs[i_perf][s]==0) ) {
				pvalues[i_perf][s]=-1;
			}		
		}
	}

}



void pvalues_BBB(double Perfs[][nb_strats_], long R, long T, double TC, double LF, double DJ_Returns[], double RF[], long IDX_BBB[][TT_], double pvalues[][nb_strats_], double Perfs_B[][B_][nb_strats_]) {

	long N=T-R+1;

	long t;
	int b, s, i_perf;
	double mean_Perfs_B;

	for(b=0; b<B_; b++) {
		DJ_B[0]=100;
		// gros bricolage!!!!! :
		//for(t=1; t<TT_; t++) {
		for(t=R-251+1; t<T+1; t++) {
			DJ_Returns_B[t]=DJ_Returns[IDX_BBB[b][t]+R-251];
			DJ_B[t]=DJ_B[t-1]*(1+DJ_Returns_B[t]);
		}
		Strats(strats_categ_, R, T, SSS_B, DJ_B, OBVI_B);
		MultiPerfs(SSS_B, R, T, TC, LF, DJ_Returns_B, RF, Perfs_tmp);
		for(i_perf=0; i_perf<nb_perf_measures_; i_perf++) {
			for(s=1; s<nb_strats_; s++) {
				Perfs_B[i_perf][b][s]=Perfs_tmp[i_perf][s];
			}
		}
		DispProgress(b);
	}

	//Compute p-values
	for(s=0; s<nb_strats_; s++) {
		for(i_perf=0; i_perf<nb_perf_measures_; i_perf++) {
			pvalues[i_perf][s]=0;
			pvalues_bis[i_perf][s]=0;
		}
	}

	for(i_perf=0; i_perf<nb_perf_measures_; i_perf++) {
		for(s=0; s<nb_strats_; s++) {
			mean_Perfs_B=0;
			for(b=0; b<B_; b++) {
				mean_Perfs_B+=Perfs_B[i_perf][b][s];
			}
			mean_Perfs_B/=B_;
			for(b=0; b<B_; b++) {
				if((Perfs_B[i_perf][b][s]-mean_Perfs_B)>Perfs[i_perf][s]) {
					pvalues[i_perf][s]++;
				} else {
					pvalues_bis[i_perf][s]++;
				}
			}
			pvalues[i_perf][s]=(pvalues[i_perf][s]<pvalues_bis[i_perf][s]) ? 2*pvalues[i_perf][s]/B_ : 2*pvalues_bis[i_perf][s]/B_;
			if( (pvalues[i_perf][s]==0) && (Perfs[i_perf][s]==0) ) {
				pvalues[i_perf][s]=-1;
			}	
		}
	}

}

void pvalues_BBB(double Perfs[][nb_strats_], long R, long T, double TC, double LF, double DJ_Returns[], double RF[], double OBVI[], long IDX_BBB[][TT_], double pvalues[][nb_strats_], double Perfs_B[][B_][nb_strats_]) {

	long N=T-R+1;

	long t;
	int b, s, i_perf;
	double mean_Perfs_B;

	double OBVI_Returns[TT_];
	double RF_Returns[TT_];
	double RF_Returns_B[TT_];

	for(b=0; b<B_; b++) {
		DJ_B[0]=100;
		OBVI_B[0]=OBVI[0];
		RF_B[0]=RF[0];
		// gros bricolage!!!!! :
		//for(t=1; t<TT_; t++) {
		for(t=R-251+1; t<T+1; t++) {
			//on pourrait utiliser des variables globales OBVI_Returns RF_Returns pour aller plus vite..
			OBVI_Returns[t]=(OBVI[t]-OBVI[t-1])/OBVI[t];
			RF_Returns[t]=(RF[t]-RF[t-1])/RF[t];

			DJ_Returns_B[t]=DJ_Returns[IDX_BBB[b][t]+R-251];
			DJ_B[t]=DJ_B[t-1]*(1+DJ_Returns_B[t]);
			OBVI_Returns_B[t]=OBVI_Returns[IDX_BBB[b][t]+R-251];
			OBVI_B[t]=OBVI_B[t-1]*(1+OBVI_Returns_B[t]);
			RF_Returns_B[t]=RF_Returns[IDX_BBB[b][t]+R-251];
			RF_B[t]=RF_B[t-1]*(1+RF_Returns_B[t]);
		}
		Strats(strats_categ_, R, T, SSS_B, DJ_B, OBVI_B);
		MultiPerfs(SSS_B, R, T, TC, LF, DJ_Returns_B, RF_B, Perfs_tmp);
		for(i_perf=0; i_perf<nb_perf_measures_; i_perf++) {
			for(s=1; s<nb_strats_; s++) {
				Perfs_B[i_perf][b][s]=Perfs_tmp[i_perf][s];
			}
		}
		DispProgress(b);
	}

	//Compute p-values
	for(s=0; s<nb_strats_; s++) {
		for(i_perf=0; i_perf<nb_perf_measures_; i_perf++) {
			pvalues[i_perf][s]=0;
			pvalues_bis[i_perf][s]=0;
		}
	}

	for(i_perf=0; i_perf<nb_perf_measures_; i_perf++) {
		for(s=0; s<nb_strats_; s++) {
			mean_Perfs_B=0;
			for(b=0; b<B_; b++) {
				mean_Perfs_B+=Perfs_B[i_perf][b][s];
			}
			mean_Perfs_B/=B_;
			for(b=0; b<B_; b++) {
				if((Perfs_B[i_perf][b][s]-mean_Perfs_B)>Perfs[i_perf][s]) {
					pvalues[i_perf][s]++;
				} else {
					pvalues_bis[i_perf][s]++;
				}
			}
			pvalues[i_perf][s]=(pvalues[i_perf][s]<pvalues_bis[i_perf][s]) ? 2*pvalues[i_perf][s]/B_ : 2*pvalues_bis[i_perf][s]/B_;
			if( (pvalues[i_perf][s]==0) && (Perfs[i_perf][s]==0) ) {
				pvalues[i_perf][s]=-1;
			}	
		}
	}

}


//double Portfolio_mu(double Perfs[], int Portfolio[]) {
//	int s;
//	double mu_tmp=0;
//	int size_port=0;
//
//	for(s=0; s<nb_strats_; s++) {
//		if(Portfolio[s]==1) {
//			mu_tmp+=exp(Perfs[s])-1;
//			size_port++;
//		}
//	}
//	mu_tmp=size_port==0 ? 0 : log(1+mu_tmp/size_port);
//	return mu_tmp;
//}

double Portfolio_mu_nber(int SSS[][N_], long R, long T, double TC, double LF, double DJ_Returns[], int NBER[], int Portfolio_exp[], int port_size_exp, int Portfolio_cont[], int port_size_cont) {

	long t;
	int s;
	int S[nb_strats_];
	int oldS[nb_strats_];
	double today_return[nb_strats_];
	double ReturnsPortfolio[TT_];
	long N=T-R+1;

	for(s=0; s<nb_strats_; s++) {
        S[s]=0;
	}

	for(t=R-1; t<T; t++) {
		if(NBER[t]==1 && port_size_exp>0) {
			for(s=0; s<nb_strats_; s++) {
				if(Portfolio_exp[s]==1) {
					oldS[s]=S[s];
					S[s]=SSS[s][t];
					today_return[s]=DJ_Returns[t+1]*S[s];
					if(S[s]!=oldS[s]){
						if(S[s]*oldS[s]==-1) {
							today_return[s]-=2*TC;
						} else if(S[s]*oldS[s]==0) {
							today_return[s]-=TC;
						}
					}
					if(S[s]==-1) {
						today_return[s]-=LF;
					}
				} else {
					today_return[s]=0;
				}
			}
			ReturnsPortfolio[t-(R-1)]=Sum(today_return, nb_strats_)/port_size_exp;
		} else if(NBER[t]==0 && port_size_cont>0) {
			for(s=0; s<nb_strats_; s++) {
				if(Portfolio_cont[s]==1) {
					oldS[s]=S[s];
					S[s]=SSS[s][t];
					today_return[s]=DJ_Returns[t+1]*S[s];
					if(S[s]!=oldS[s]){
						if(S[s]*oldS[s]==-1) {
							today_return[s]-=2*TC;
						} else if(S[s]*oldS[s]==0) {
							today_return[s]-=TC;
						}
					}
					if(S[s]==-1) {
						today_return[s]-=LF;
					}
				} else {
					today_return[s]=0;
				}
			}
			ReturnsPortfolio[t-(R-1)]=Sum(today_return, nb_strats_)/port_size_cont;
		} else {
			ReturnsPortfolio[t-(R-1)]=0;
		}
	}

	return Sum(ReturnsPortfolio, N)/N;;

}



void Best_portfolio(double Perfs[], int size, int Portfolio[]) {
	int s;
	strat_struct my_table[nb_strats_];

	for(s=0; s<nb_strats_; s++) {
		Portfolio[s]=0;
	}

	for(s=0; s<nb_strats_; s++) {
		my_table[s].idx=s;
		my_table[s].perf=Perfs[s];
        my_table[s].pvalue=10;
	}
	qsort(my_table, nb_strats_, sizeof(strat_struct), my_compare_strat_struct_perf_dec);

	for(s=0; s<size; s++) {
		Portfolio[my_table[s].idx]=1;
	}
}


void Sort_by_categories(int Portfolio[], int NB_rules_by_categories[]) {

	int s;

	int nb_FR=0;
	int nb_MA=0;
	int nb_SR=0;
	int nb_CB=0;
	int nb_OBV=0;

	for(s=0; s<nb_strats_; s++) {
		if(Portfolio[s]==1) {
			if(s<497) {
				nb_FR++;
			} else if(497<=s && s<2546) {
				nb_MA++;
			} else if(2546<=s && s<3766) {
				nb_SR++;
			} else if(3766<=s && s<5806) {
				nb_CB++;
			} else {
				nb_OBV++;
			}
		}
	}

	NB_rules_by_categories[0]=nb_FR;
	NB_rules_by_categories[1]=nb_MA;
	NB_rules_by_categories[2]=nb_SR;
	NB_rules_by_categories[3]=nb_CB;
	NB_rules_by_categories[4]=nb_OBV;

}

void Sort_by_horizon(int Portfolio[], int SLOW[], int NB_rules_by_horizon[]) {

	int s;

	int nb_slow=0;
	int nb_fast=0;

	for(s=0; s<nb_strats_; s++) {
		if(Portfolio[s]==1 && SLOW[s]==1) {
			nb_slow++;
		} else if (Portfolio[s]==1 && SLOW[s]==0) {
			nb_fast++;
		}
	}
	NB_rules_by_horizon[0]=nb_slow;
	NB_rules_by_horizon[1]=nb_fast;
}


void Compute_nb_trades(int SSS[][N_], long NB_trades[]) {

	int s;
	long t;

	long nb_trades;

	for(s=0; s<nb_strats_; s++) {
		nb_trades=0;
		if(SSS[s][0]!=0) {
			nb_trades++;
		}
		for(t=1; t<N_; t++) {
			if(SSS[s][t]!=SSS[s][t-1]) {
				nb_trades++;
			}
		}
		NB_trades[s]=nb_trades;
	}

}


double DJ_return_exp(long R, long T, int nber, double DJ_Returns[], int NBER[]) {

	double DJ_return_exp=0;
	long N=0;
	long t;

	for(t=R; t<=T; t++) {
		if(NBER[t]==nber) {
            DJ_return_exp+=DJ_Returns[t];
			N++;
		}
	}
	DJ_return_exp= (N==0) ? 0 : DJ_return_exp/N;
	return DJ_return_exp;

}


