#include <cstdlib>
#include <cmath>

#include <string>
#include <fstream>
#include <iostream>
#include <sstream>

#include "Umgebung.h"

#include "FilterRules.h"
#include "MovingAverages.h"
#include "SupportResistance.h"
#include "ChannelBreakouts.h"
#include "Tools.h"

using namespace std;

//Filter Rules:
const int fr_nx=24;
const double fr_x[]={0.005, 0.01, 0.015, 0.02, 0.025, 0.03, 0.035, 0.04, 0.045, 0.05,
	0.06, 0.07, 0.08, 0.09, 0.1, 0.12, 0.14, 0.16, 0.18, 0.2, 0.25, 0.3, 0.4, 0.5};
const int fr_ny=12;
const double fr_y[]={0.005, 0.01, 0.015, 0.02, 0.025, 0.03, 0.04, 0.05, 0.075,
	0.1, 0.15, 0.2};
//for alternative def of extrema:
const int fr_ne=8;
const int fr_e[]={1, 2, 3, 4, 5, 10, 15, 20};
//constant holding period
const int fr_nc=4;
const int fr_c[]={5, 10, 25, 50};

//Moving Averages:
const int ma_nn=16;
const int ma_n[]={1, 2, 5, 10, 15, 20, 25, 30, 40, 50, 75, 100, 125, 150, 200, 250};
//percentage band:
const int ma_nb=9;
const double ma_b[]={0, 0.001, 0.005, 0.01, 0.015, 0.02, 0.03, 0.04, 0.05}; //0 for no filter
//time delay:
const int ma_nd=4;
const int ma_d[]={2, 3, 4, 5};
//constant holding period:
const int ma_nc=4;
const int ma_c[]={5, 10, 25, 50};
//for the 9 additional rules:
const int ma_nnf=3;
const int ma_nf[]={1, 2, 5};
const int ma_nns=3;
const int ma_ns[]={50, 150, 200};

//Support and Resistance:
//nb of days for the extremum:
const int sr_nn=10;
const int sr_n[]={5, 10, 15, 20, 25, 50, 100, 150, 200, 250};
//for alternative def of extrema:
const int sr_ne=10;
const int sr_e[]={2, 3, 4, 5, 10, 20, 25, 50, 100, 200};
//percentage band:
const int sr_nb=9;
const double sr_b[]={0, 0.001, 0.005, 0.01, 0.015, 0.02, 0.03, 0.04, 0.05};
//time delay filter:
const int sr_nd=4;
const int sr_d[]={2, 3, 4, 5};
//constant holding period:
const int sr_nc=5;
const int sr_c[]={1, 5, 10, 25, 50};

//Channel Breakouts:
//number of days for the channel:
const int cb_nn=10;
const int cb_n[]={5, 10, 15, 20, 25, 50, 100, 150, 200, 250};
//diff between high and low for the channel:
const int cb_nx=8;
const double cb_x[]={0.005, 0.01, 0.02, 0.03, 0.05, 0.075, 0.1, 0.15};
//percentage band:
const int cb_nb=9;
const double cb_b[]={0, 0.001, 0.005, 0.01, 0.015, 0.02, 0.03, 0.04, 0.05};
//constant holding period:
const int cb_nc=4;
const int cb_c[]={5, 10, 25, 50};

void Strats(int strats_categ, long R, long T, int SSS[][N_], double DJ[], double OBVI[]) {

	int S;
	long t_holding_period;	//keeps track of start of holding period
	long t_extrema;			//keeps track of where to look for extrema
	long t;
	int i, j, k, l;
	stringstream ss;
	long count=0;

	if(strats_categ==0 || strats_categ==1 || strats_categ==9) {
		//Filter rule 1:
		//first def of subsequent high/low
		for(i=0; i<fr_nx; i++) {
			S=0;
			t_extrema=R-2;
			for(t=R-1; t<T; t++) {
		  		S=FilterRule1(S, t, &t_extrema, fr_x[i], DJ);
				SSS[count][t]=S;
			}
			count++;
			//DispProgress(count);
		}

		//Filter rule 2:
		//alternative def of subsequent high/low, no neutral position
		for(i=0; i<fr_nx; i++) {
			for(j=0; j<fr_ne; j++) {
				S=0;
				t_extrema=R-2;
				for(t=R-1; t<T; t++) {
					S=FilterRule2(S, t, &t_extrema, fr_x[i], fr_e[j], DJ);
					SSS[count][t]=S;
				}
				count++;
				//DispProgress(count);
			}
		}

		//Filter rule 3:
		//constant holding period, first def of subsequent high/low
		for(i=0; i<fr_nx; i++) {
			for(j=0; j<fr_nc; j++) {
				t_extrema=R-2;
				t_holding_period=R-fr_c[fr_nc-1]-10;
				for(t=R-1; t<T; t++) {
		  			S=FilterRule3(S, t, &t_extrema, &t_holding_period, fr_x[i], fr_c[j], DJ);
		  			SSS[count][t]=S;
				}
				count++;
				//DispProgress(count);
			}
		}
		
		//Filter rule 4:
		//first def of subsequent high/low, neutral position
		for(i=0; i<fr_nx; i++) {
			for(j=0; j<fr_ny; j++) {
				if(fr_y[j]<fr_x[i]) {
					S=0;
					t_extrema=R-2;
					for(t=R-1; t<T; t++) {
		  				S=FilterRule4(S, t, &t_extrema, fr_x[i], fr_y[j], DJ);
		  				SSS[count][t]=S;
					}
					count++;
					//DispProgress(count);
				}
			}
		}
	}

	if(strats_categ==0 || strats_categ==2 || strats_categ==9) {
		//Moving average 1:
		//two MAs, constant holding period, percentage band filter:
		for(i=1; i<ma_nn; i++) { //slow
			for(j=0; j<i; j++) { //fast
				for(k=0; k<ma_nc; k++) {
					S=0;
					t_holding_period=R-1-ma_c[ma_nc-1]-10;
					for(t=R-1; t<T; t++) {
						S=MovingAverage1(S, t, &t_holding_period, ma_n[i], ma_n[j], ma_c[k], 0, DJ);
						SSS[count][t]=S;
					}
					count++;
					//DispProgress(count);
				}
				for(l=0; l<ma_nb; l++) {
					S=0;
					t_holding_period=R-1-ma_c[ma_nc-1]-10;
					for(t=R-1; t<T; t++) {
						S=MovingAverage1(S, t, &t_holding_period, ma_n[i], ma_n[j], 1, ma_b[l], DJ);
						SSS[count][t]=S;
					}
					count++;
					//DispProgress(count);
				}
			}
		}

		//Moving average 1:
		//9 additional rules:
		for(i=0; i<ma_nns; i++) {
			for(j=0; j<ma_nnf; j++) {
				int c=10;
				double b=0.01;
				S=0;
				t_holding_period=R-1-c-10;
				for(t=R-1; t<T; t++) {
					S=MovingAverage1(S, t, &t_holding_period, ma_ns[i], ma_nf[j], c, b, DJ);
					SSS[count][t]=S;
				}
				count++;
				//DispProgress(count);
			}
		}

		//Moving average 2:
		//two MAs, time delay
		for(i=1; i<ma_nn; i++) { //slow
			for(j=0; j<i; j++) { //fast
				for(k=0; k<ma_nd; k++) {
					for(t=R-1; t<T; t++) {
						if(t>=(R-1+ma_d[k]-1))
							S=MovingAverage2(t, ma_n[i], ma_n[j], ma_d[k], DJ);
						else
							S=0;
						SSS[count][t]=S;
					}
					count++;
					//DispProgress(count);
				}
			}
		}
	}

	if(strats_categ==0 || strats_categ==3 || strats_categ==9) {
		//Support and Resistance 1:
		//first defintion of extrema, constant holding period, percentage band filter
		for(i=0; i<sr_nn; i++) {
			for(j=0; j<sr_nb; j++) {
				for(k=0; k<sr_nc; k++) {
					S=0;
					t_holding_period=R-1-sr_c[sr_nc-1]-10;
					for(t=R-1; t<T; t++) {
						S=SupportResistance1(S, t, &t_holding_period, sr_n[i], sr_b[j], sr_c[k], DJ);
						SSS[count][t]=S;
					}
					count++;
					//DispProgress(count);
				}
			}
		}

		//Support and Resistance 2:
		//first def of extrema, constant holding period, time delay filter
		for(i=0; i<sr_nn; i++) {
			for(j=0; j<sr_nd; j++) {
				for(k=1; k<sr_nc; k++) {
					S=0;
					t_holding_period=R-1-sr_c[sr_nc-1]-10;
					for(t=R-1; t<T; t++) {
						if(t>=(R-1+sr_d[j]-1))
							S=SupportResistance2(S, t, &t_holding_period, sr_n[i], sr_d[j], sr_c[k], DJ);
						else
							S=0;					
						SSS[count][t]=S;
					}
					count++;
					//DispProgress(count);
				}
			}
		}

		//Support and Resistance 1b:
		//alternative defintion of extrema, constant holding period, percentage band filter
		for(i=0; i<sr_ne; i++) {
			for(j=0; j<sr_nb; j++) {
				for(k=0; k<sr_nc; k++) {
					S=0;
					t_holding_period=R-1-sr_c[sr_nc-1]-10;
					t_extrema=R-2;
					for(t=R-1; t<T; t++) {
						S=SupportResistance1b(S, t, &t_extrema, &t_holding_period, sr_e[i], sr_b[j], sr_c[k], DJ);
						SSS[count][t]=S;
					}
					count++;
					//DispProgress(count);
				}
			}
		}

		//Support and Resistance 2b:
		//alternative def of extrema, constant holding period, time delay filter
		for(i=0; i<sr_ne; i++) {
			for(j=0; j<sr_nd; j++) {
				for(k=1; k<sr_nc; k++) {
					S=0;
					t_holding_period=R-1-sr_c[sr_nc-1]-10;
					t_extrema=R-2;
					for(t=R-1; t<T; t++) {
						if(t>=(R-1+sr_d[j]-1))
							S=SupportResistance2b(S, t, &t_extrema, &t_holding_period, sr_e[i], sr_d[j], sr_c[k], DJ);
						else
							S=0;
						SSS[count][t]=S;
					}
					count++;
					//DispProgress(count);
				}
			}
		}
	}


	if(strats_categ==0 || strats_categ==4 || strats_categ==9) {
		// Channel Breakouts:
		for(i=0; i<cb_nn; i++) {
			for(j=0; j<cb_nx; j++) {
				for(k=0; k<cb_nc; k++) {
					for(l=0; l<cb_nb; l++) {
						if(cb_b[l]<cb_x[j]) {
							S=0;
							t_holding_period=R-1-cb_c[cb_nc-1]-10;
							for(t=R-1; t<T; t++) {
								S=ChannelBreakouts(S, t, &t_holding_period, cb_n[i], cb_x[j], cb_c[k], cb_b[l], DJ);
								SSS[count][t]=S;
							}
							count++;
							//DispProgress(count);
						}
					}
				}
			}
		}
	}


	if(strats_categ==0 || strats_categ==5) {
		//On-Balance Volume - Moving average 1:
		//two MAs, constant holding period, percentage band filter:
		for(i=1; i<ma_nn; i++) { //slow
			for(j=0; j<i; j++) { //fast
				for(k=0; k<ma_nc; k++) {
					S=0;
					t_holding_period=R-1-ma_c[ma_nc-1]-10;
					for(t=R-1; t<T; t++) {
						S=MovingAverage1(S, t, &t_holding_period, ma_n[i], ma_n[j], ma_c[k], 0, OBVI);
						SSS[count][t]=S;
					}
					count++;
					//DispProgress(count);
				}
				for(l=0; l<ma_nb; l++) {
					S=0;
					t_holding_period=R-1-ma_c[ma_nc-1]-10;
					for(t=R-1; t<T; t++) {
						S=MovingAverage1(S, t, &t_holding_period, ma_n[i], ma_n[j], 1, ma_b[l], OBVI);
						SSS[count][t]=S;
					}
					count++;
					//DispProgress(count);
				}
			}
		}

		//On-Balance Volume - Moving average 2:
		//two MAs, time delay
		for(i=1; i<ma_nn; i++) { //slow
			for(j=0; j<i; j++) { //fast
				for(k=0; k<ma_nd; k++) {
					for(t=R-1; t<T; t++) {
						if(t>=(R-1+ma_d[k]-1))
							S=MovingAverage2(t, ma_n[i], ma_n[j], ma_d[k], OBVI);
						else
							S=0;
						SSS[count][t]=S;
					}
					count++;
					//DispProgress(count);
				}
			}
		}
	}

}
	
	
	




